!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Information System (LIS) v7.1
!
! Copyright (c) 2015 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
module LIS_perturbMod

!BOP
!
! !MODULE: LIS_perturbMod
!
! !DESCRIPTION:
!  The code in this file provides abstractions to manage different 
!  perturbation implementations. This includes methods to perturb
!  forcing, observations, and land surface states. 
!  
! !REVISION HISTORY:
!
!  25 Jun 2006: Sujay Kumar; Initial implementation
!  24Nov10    Sujay Kumar; Added support for time varying 
!                          perturbations
!
!EOP
  use ESMF
  use LIS_coreMod
  use LIS_timeMgrMod

  implicit none
  
  PRIVATE
!-----------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!-----------------------------------------------------------------------------
  public :: LIS_perturb_init    ! initialization for perturbation routines
  public :: LIS_perturb_readrestart ! read perturbation restart files
  public :: LIS_perturb_writerestart ! write perturbation restart files
  public :: LIS_readPertAttributes !read perturbation attributes
!-----------------------------------------------------------------------------
! !PUBLIC TYPES:
!-----------------------------------------------------------------------------

!EOP

  type, public :: pert_dec_type
     character*40,allocatable :: vname(:)     !name of the observation variables
     integer     ,allocatable :: perttype(:)  !type of perturbation (0-additive, 1-multiplicative)
     real        ,allocatable :: ssdev(:) !standard deviation of perturbation
     real        ,allocatable :: stdmax(:) !standard normal max for the perturbation distribution
     integer     ,allocatable :: zeromean(:) !flag to specify if zero mean is to be ensured
     real        ,allocatable :: tcorr(:) !temporal correlation scale (seconds)
     real        ,allocatable :: xcorr(:) !horizontal x-correlation scale
     real        ,allocatable :: ycorr(:) !horizontal y-correlation scale
     real        ,allocatable :: ccorr(:,:) !cross correlations (specified as a matrix)
  end type pert_dec_type 

contains
 
!BOP
! 
! !ROUTINE: LIS_perturb_init
! \label{LIS_perturb_init}
! 
! !INTERFACE:  
  subroutine LIS_perturb_init
! 
! !DESCRIPTION: 
! 
! The routines invoked are: 
! \begin{description}
!  \item[LIS\_perturb\_plugin](\ref{LIS_perturb_plugin}) \newline
!    sets up the function table registries for implemented
!    perturbation algorithms
!  \end{description}
! 
! !USES: 
    use LIS_logMod,             only : LIS_verify
    use LIS_timeMgrMod,         only : LIS_registerAlarm
    use LIS_perturb_pluginMod
!EOP
    integer                   :: rc

    call LIS_perturb_plugin

    if(LIS_rc%nperts.gt.0) then 
       call LIS_registerAlarm("LIS pert restart alarm",&
            1800.0 ,&  ! 1800 must be modified with the min perturbation timestep
            LIS_rc%pertrestartInterval)
    endif

  end subroutine LIS_perturb_init
!BOP
! 
! !ROUTINE: LIS_perturb_readrestart
! \label{LIS_perturb_readrestart}
! 
! !INTERFACE: 
  subroutine LIS_perturb_readrestart
! !USES: 
! !ARGUMENTS:     

! 
! !DESCRIPTION: 
! 
!EOP
    if (LIS_rc%pertrestart.eq."restart".and.LIS_rc%nperts.gt.0) then 
       call readpertrestart(trim(LIS_rc%perturb_forcing)//char(0))
    endif

  end subroutine LIS_perturb_readrestart
!BOP
! 
! !ROUTINE: LIS_perturb_writerestart
! \label{LIS_perturb_writerestart}
! 
! !INTERFACE: 
  subroutine LIS_perturb_writerestart(n)
! !USES: 
! !ARGUMENTS:     
    integer, intent(IN) :: n 
! 
! !DESCRIPTION: 
! 
!EOP
    integer           :: rc
    logical           :: alarmCheck

    if(LIS_rc%nperts.gt.0) then 
       
       alarmCheck = LIS_isAlarmRinging(LIS_rc, "LIS pert restart alarm")

       if(alarmCheck) then 
!  NOTE: 
!  A single restart file will be written for forcing, state and observation
!  perturbations. Here we assume that the same perturbation algorithm will be 
!  used for all three. 
          if(LIS_rc%perturb_forcing.ne."none") then
             call writepertrestart(trim(LIS_rc%perturb_forcing)//char(0), n)
          endif
       endif
    endif

  end subroutine LIS_perturb_writerestart

!BOP
! !ROUTINE: LIS_readPertAttributes
! \label{LIS_readPertAttributes}
!
! !REVISION HISTORY:
!  14 Oct 2006: Sujay Kumar; Initial Specification
!
! !INTERFACE:
  subroutine LIS_readPertAttributes(nvars, fname,pert_struc)
! !USES: 
    use LIS_logMod,    only : LIS_logunit, & 
         LIS_getNextUnitNumber, LIS_releaseUnitNumber

    implicit none
! !ARGUMENTS: 
    integer             :: nvars
    character(len=*)    :: fname
    type(pert_dec_type) :: pert_struc
! 
! !DESCRIPTION: 
!  This routine reads the observation perturbation attributes for each
!  observation type. The format of the perturbation attributes file
!  is: 
!
!\begin{verbatim}
!  <Variable name>  
!  <perturbation type> <perturbation standard deviation type>
!  <standard deviation> <coeff standard deviation (optional, only if 
!  perturbation standard deviation type is > 0) <standard normal max> (contd.)
!  <ensure zeromean?> <temporal correlation scale> (contd.)
!  <x correlation scale>  <y correlation scale> <cross correlations> 
!\end{verbatim}
!
!  The arguments are: 
!  \begin{description}
!   \item[n]           index of nest
!   \item[vname]       name of the observation variables
!   \item[perttype]    type of perturbation (0-additive, 1-multiplicative)
!   \item[ssdev]       standard deviation of perturbation
!   \item[stdmax]      standard normal max for the perturbation distribution
!   \item[zeromean]    flag to specify if zero mean is to be ensured
!   \item[tcorr]       temporal correlation scale (seconds)
!   \item[xcorr]       horizontal x-correlation scale
!   \item[ycorr]       horizontal y-correlation scale
!   \item[ccorr]       cross correlations
!  \end{description}
!EOP
    integer            :: i,j
    integer            :: ftn 


    write(LIS_logunit,*) 'Opening perturbation attributes for observations ',&
         fname
    ftn = LIS_getNextUnitNumber()
    open(ftn,file=fname,status='old')
    read(ftn,*)
    
    do i=1,nvars

       read(ftn,fmt='(a40)') pert_struc%vname(i)
       read(ftn,*) pert_struc%perttype(i) , & 
            pert_struc%ssdev(i), pert_struc%stdmax(i), &
            pert_struc%zeromean(i), &
            pert_struc%tcorr(i), pert_struc%xcorr(i), pert_struc%ycorr(i), &
            (pert_struc%ccorr(i,j),j=1,nvars)
       
       write(LIS_logunit,*) pert_struc%vname(i), pert_struc%perttype(i), &
            pert_struc%ssdev(i),pert_struc%stdmax(i),&
            pert_struc%zeromean(i), &
            pert_struc%tcorr(i), pert_struc%xcorr(i),pert_struc%ycorr(i), &
            (pert_struc%ccorr(i,j),j=1,nvars)
              
    enddo

    call LIS_releaseUnitNumber(ftn)

  end subroutine LIS_readPertAttributes

end module LIS_perturbMod
